@@ -55,11 +55,15 @@ with ignoring(ImportError):
     from .backends import sql_csv
 with ignoring(ImportError):
     from .backends.bokeh import ColumnDataSource
+with ignoring(ImportError):
+    from .backends.spark import RDD
+with ignoring(ImportError):
+    from .backends.sparksql import SchemaRDD
 
 
 restart_ordering()  # Restart multipledispatch ordering and do ordering
 
-__version__ = '0.2.1'
+__version__ = '0.2.2'
 
 
 def test(verbose=False, junitfile=None, exit=False):
