@@ -9,12 +9,13 @@ import os
 from .csv import CSV
 from .json import JSON, JSONLines
 from .text import TextFile
+import pandas as pd
 import uuid
 import datashape
 import sqlalchemy as sa
 from datashape import discover, dshape
 from datashape import coretypes as ct
-from collections import namedtuple
+from collections import namedtuple, Iterator
 from contextlib import contextmanager
 from .ssh import SSH, _SSH
 from .sql import metadata_of_engine, sa
@@ -22,6 +23,7 @@ from ..utils import tmpfile, sample, ignoring, raises
 from ..temp import Temp
 from ..append import append
 from ..convert import convert
+from ..chunks import chunks
 from ..resource import resource
 from ..directory import _Directory, Directory
 from ..compatibility import unicode
@@ -70,17 +72,26 @@ def HDFS(cls):
     return type('HDFS(%s)' % cls.__name__, (_HDFS, cls), {'subtype':  cls})
 
 
-@sample.register(HDFS(CSV))
+@sample.register(_HDFS)
 @contextmanager
 def sample_hdfs_csv(data, length=10000):
     sample = data.hdfs.read_file(data.path.lstrip('/'), length=length)
-    with tmpfile('.csv') as fn:
+    with tmpfile(data.canonical_extension) as fn:
         with open(fn, 'w') as f:
             f.write(sample)
 
         yield fn
 
 
+@discover.register(HDFS(JSON))
+@discover.register(HDFS(JSONLines))
+@discover.register(HDFS(TextFile))
+def discover_hdfs_file(data, **kwargs):
+    with sample(data) as fn:
+        result = discover(data.subtype(fn, **kwargs))
+    return result
+
+
 @discover.register(HDFS(CSV))
 def discover_hdfs_csv(data, length=10000, **kwargs):
     with sample(data, length=length) as fn:
@@ -284,12 +295,7 @@ def create_new_hive_table_from_csv(tbl, data, dshape=None, path=None, **kwargs):
 
     Actually executes command.
     """
-    if isinstance(data, _SSH):
-        table_type = None
-    elif isinstance(data, _HDFS):
-        table_type = 'EXTERNAL'
-    else:
-        table_type = None
+    table_type = 'EXTERNAL'
 
     if not dshape:
         dshape = discover(data)
@@ -313,6 +319,14 @@ def create_new_hive_table_from_csv(tbl, data, dshape=None, path=None, **kwargs):
     return tbl2
 
 
+@append.register(TableProxy, (HDFS(CSV), Temp(HDFS(CSV))))
+def create_new_hive_table_from_csv_file(tbl, data, dshape=None, path=None, **kwargs):
+    raise ValueError(
+        "Can not create a new Hive table from a single CSV file on HDFS.\n"
+        "Instead try loading a complete directory or base your data outside of"
+        " HDFS")
+
+
 @append.register(TableProxy, (SSH(CSV), SSH(Directory(CSV))))
 def append_remote_csv_to_new_table(tbl, data, dshape=None, **kwargs):
     if not dshape:
@@ -361,6 +375,21 @@ def append_remote_csv_to_table(tbl, csv, **kwargs):
     return tbl
 
 
+@append.register(sa.Table, (HDFS(CSV), HDFS(Directory(CSV)), Temp(HDFS(CSV))))
+def append_hdfs_csv_to_table(tbl, csv, **kwargs):
+    """
+    Load Remote data into existing Hive table
+    """
+    if tbl.bind.dialect.name != 'hive':
+        raise NotImplementedError("Don't know how to migrate csvs on remote "
+                  "disk to database of dialect %s" % tbl.engine.dialect.name)
+
+    statement =('LOAD DATA INPATH "%(path)s" INTO TABLE %(tablename)s'
+                 % {'path': csv.path, 'tablename': tbl.name})
+    with tbl.bind.connect() as conn:
+        conn.execute(statement)
+    return tbl
+
 @append.register(TextFile, HDFS(TextFile))
 @append.register(JSONLines, HDFS(JSONLines))
 @append.register(JSON, HDFS(JSON))
@@ -421,7 +450,7 @@ def dialect_of(data, **kwargs):
 
         # Get sample text
         with open(data.path, 'r') as f:
-            text = f.read(1000)
+            text = f.read()
 
         result = dict()
 
@@ -467,6 +496,15 @@ def resource_hdfs(uri, **kwargs):
     return HDFS(subtype)(path, **kwargs)
 
 
+@append.register(HDFS(TextFile), (Iterator, object))
+@append.register(HDFS(JSONLines), (Iterator, object))
+@append.register(HDFS(JSON), (list, object))
+@append.register(HDFS(CSV), (chunks(pd.DataFrame), pd.DataFrame, object))
+def append_object_to_hdfs(target, source, **kwargs):
+    tmp = convert(Temp(target.subtype), source, **kwargs)
+    return append(target, tmp, **kwargs)
+
+
 @append.register(HDFS(TextFile), SSH(TextFile))
 @append.register(HDFS(JSONLines), SSH(JSONLines))
 @append.register(HDFS(JSON), SSH(JSON))
