@@ -7,7 +7,7 @@ from datashape import discover
 import re
 import uuid
 
-from ..directory import _Directory, Directory
+from ..directory import Directory
 from ..utils import keywords, tmpfile, sample, ignoring
 from ..resource import resource
 from ..append import append
@@ -122,7 +122,6 @@ def sample_ssh(data, lines=500):
         with open(fn, 'w') as f:
             for line in take(lines, data.lines()):
                 f.write(line)
-                f.write('\n')
         yield fn
 
 
