@@ -15,9 +15,11 @@ from into.backends.ssh import SSH, resource, ssh_pattern, sftp, drop, connect
 from into.backends.csv import CSV
 from into import into, discover, CSV, JSONLines, JSON, convert
 from into.temp import _Temp, Temp
-from into.compatibility import PY3, skipif, ON_TRAVIS_CI
+from into.compatibility import ON_TRAVIS_CI
 import socket
 
+skipif = pytest.mark.skipif
+
 try:
     ssh = connect(hostname='localhost')
     ssh.close()
@@ -157,8 +159,8 @@ def test_convert_through_temporary_local_storage():
         assert (into(np.ndarray, sjson) == into(np.ndarray, df)).all()
 
 
-@pytest.mark.skipif(ON_TRAVIS_CI,
-                    reason='Strange hanging on travis for python33')
+@skipif(ON_TRAVIS_CI and sys.version_info[:2] == (3, 3),
+        reason='Strange hanging on travis for python33')
 def test_ssh_csv_to_s3_csv():
     # for some reason this can only be run in the same file as other ssh tests
     # and must be a Temp(SSH(CSV)) otherwise tests above this one fail
@@ -169,3 +171,15 @@ def test_ssh_csv_to_s3_csv():
         with s3_bucket('.csv') as b:
             result = into(b, remote)
             assert discover(result) == discover(resource(b))
+
+
+@skipif(ON_TRAVIS_CI and sys.version_info[:2] == (3, 3),
+        reason='windows does not have an SSH daemon on localhost')
+def test_s3_to_ssh():
+    pytest.importorskip('boto')
+
+    tips_uri = 's3://nyqpug/tips.csv'
+    with tmpfile('.csv') as fn:
+        result = into(Temp(SSH(CSV))(fn, hostname='localhost'), tips_uri)
+        assert into(list, result) == into(list, tips_uri)
+        assert discover(result) == discover(resource(tips_uri))
